using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.Events;
using System.Linq;

namespace Mod
{
    public class Residue : Liquid
    {
        public Residue()
        {
            Color = new Color(0.751f, 0.752f, 0.753f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }



    public class PermanentDeathPoison : TemporaryBodyLiquid
    {
        public const string ID = "PERMANENT DEATH POISON";
        public PermanentDeathPoison() => this.Color = new Color(1f, 0.29f, 0.25f);

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<VeryDead>();
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<MediGun>());
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        { 
        }
    }
    public class VeryDead : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }
        public void Update()
        {            
            limb.CirculationBehaviour.IsPump = false;
        }
    }

    public class GillsSerum : TemporaryBodyLiquid
    {
        public const string ID = "GILLS SERUM";
        public GillsSerum() => this.Color = new Color(0f, 0.61f, 0.89f);

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<Gills>();
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        { 
        }
    }
    public class Gills : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }
        public void Update()
        {            
            if (limb.SpeciesIdentity == "Android" || limb.RoughClassification != LimbBehaviour.BodyPart.Head)
                return;
            limb.Person.OxygenLevel = 1f;
        }
    }

    public class ComaPoison : TemporaryBodyLiquid
    {
        public const string ID = "COMA POISON";
        public ComaPoison() => this.Color = new Color(0.66f, 0.52f, 0.33f);

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<HeartBeatGoBrrr>());
            limb.gameObject.AddComponent<Coma>();
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {
        }
    }
    public class Coma : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }
        public void Update()
        {
            limb.Person.Consciousness = 0f;       
        }
    }

    public class EnhancedAdrenaline : TemporaryBodyLiquid
    {
        public EnhancedAdrenaline()
        {
            Color = new Color(0.21f, 0.12f, 0.37f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<HeartBeatGoBrrr>();
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<Coma>());
            limb.Person.Consciousness = 1f;
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }
    public class HeartBeatGoBrrr : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }
        public void Update()
        {
            limb.Person.AdrenalineLevel = 1f; 
        }
    }

    public class EnhancedCoagulationSerum : TemporaryBodyLiquid
    {
        public EnhancedCoagulationSerum()
        {
            Color = new Color(0.75f, 1f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<WoundHealer>();
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }
    public class WoundHealer : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }
        public void Update()
        {
            if (limb.SpeciesIdentity == "Android")
                return;
            limb.CirculationBehaviour.HealBleeding();
        }
    }

    public class AeternavitaSerum : TemporaryBodyLiquid
    {
        public AeternavitaSerum()
        {
            Color = new Color(0.35f, 1f, 0.54f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.BreakingThreshold *= 1000f;
            if(limb.SpeciesIdentity == "Android")
                return;
            limb.gameObject.AddComponent<MediGun>();
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<VeryDead>());
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }
    public class MediGun : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }

        public void Update()
        {
            limb.LungsPunctured = false;
            limb.HealBone();
            limb.Health = limb.InitialHealth;
            limb.Numbness = 0.0f;
            limb.CirculationBehaviour.HealBleeding();
            limb.CirculationBehaviour.IsPump = limb.CirculationBehaviour.WasInitiallyPumping;
            limb.CirculationBehaviour.BloodFlow = 1f;
            double num = (double) limb.CirculationBehaviour.AddLiquid(limb.GetOriginalBloodType(), Mathf.Max(0.0f, 1f - limb.CirculationBehaviour.GetAmount(limb.GetOriginalBloodType())));
            limb.BruiseCount = (ushort) 0;
            limb.CirculationBehaviour.GunshotWoundCount = (ushort) 0;
            limb.CirculationBehaviour.StabWoundCount = (ushort) 0;
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Head)
            {
                limb.Person.Consciousness = 1f;
                limb.Person.ShockLevel = 0.0f;
                limb.Person.PainLevel = 0.0f;
                limb.Person.OxygenLevel = 1f;
                limb.Person.AdrenalineLevel = 1f;
            }
            limb.PhysicalBehaviour.BurnProgress -= Time.deltaTime * 0.16f;
            limb.SkinMaterialHandler.AcidProgress -= Time.deltaTime * 0.8f;
            limb.SkinMaterialHandler.RottenProgress -= Time.deltaTime * 0.8f;
            for (int index = 0; index < limb.SkinMaterialHandler.bulletHolePoints.Length; ++index)
            {
                limb.SkinMaterialHandler.bulletHolePoints[index].z = Mathf.Max(limb.SkinMaterialHandler.bulletHolePoints[index].z, 0.05f);
                limb.SkinMaterialHandler.bulletHolePoints[index].z += Time.deltaTime * 0.5f;
            }
            limb.SkinMaterialHandler.Sync();
            for (int index = 0; index < limb.SkinMaterialHandler.damagePoints.Length; ++index)
            {
                limb.SkinMaterialHandler.damagePoints[index].z += Time.deltaTime * 0.5f;
            }
            limb.SkinMaterialHandler.Sync();
        }
    }

    public class InternalBleedingPoison : TemporaryBodyLiquid
    {
        public InternalBleedingPoison()
        {
            Color = new Color(1f, 0.43f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.InternalBleedingIntensity += 0.4f * UnityEngine.Random.value;    
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class BleedingPoison : TemporaryBodyLiquid
    {
        public BleedingPoison()
        {
            Color = new Color(1f, 0.22f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.BleedingRate += 1.5f;
            }            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.BleedingRate += 1.5f;
            }
        }
    }

    public class BoneHealingSerum : TemporaryBodyLiquid
    {
        public BoneHealingSerum()
        {
            Color = new Color(0f, 0.55f, 0.22f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.HealBone();
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class VitamineDSerum : TemporaryBodyLiquid
    {
        public VitamineDSerum()
        {
            Color = new Color(0f, 1f, 0.4f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.Limb.HealBone();
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }        
}