using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class NeutralizedReanimationAgent : Liquid
    {
        public NeutralizedReanimationAgent()
        {
            Color = new Color(0.44f, 0.5f, 0.36f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }

    public class PartialReanimationAgentAntidote : TemporaryBodyLiquid
    {
        public PartialReanimationAgentAntidote()
        {
            Color = new Color(0.8f, 1f, 0.5f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container is CirculationBehaviour circ && circ.Limb.IsZombie == false)
            {
                circ.Limb.gameObject.AddComponent<PartialAntiZombieShieldMyGodWhyIsThisSoLong>();
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }
    public class PartialAntiZombieShieldMyGodWhyIsThisSoLong : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }

        public void Update()
        {
            limb.IsZombie = false;
            limb.SkinMaterialHandler.RottenProgress -= Time.deltaTime * 0.2f;
        }
    }

    public class ReanimationAgentAntidote : TemporaryBodyLiquid
    {
        public ReanimationAgentAntidote()
        {
            Color = new Color(0.7f, 1f, 0.25f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.Limb.IsZombie = false;
                circ.Limb.SkinMaterialHandler.RottenProgress -= Time.deltaTime * 0.4f;
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }

    public class ElectricalFluid : Liquid
    {
        public ElectricalFluid()
        {
            Color = new Color(0f, 1f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            PhysicalBehaviour phys;
            phys = container.gameObject.GetComponent<PhysicalBehaviour>();
            var ChargeProportion = container.GetAmount((Liquid) this) / 1.4f;
            if(container.gameObject.GetComponent<StormBaanBehaviour>())
                return;
            phys.charge = 50f * ChargeProportion;
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }

    public class DecimatorFluid : Liquid
    {
        public DecimatorFluid()
        {
            Color = new Color(0f, 0.1f, 0.1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            PhysicalBehaviour phys;
            phys = container.gameObject.GetComponent<PhysicalBehaviour>();
            if(container is CirculationBehaviour circ)
            {
                UnityEngine.Object.Destroy((UnityEngine.Object) phys.transform.root.gameObject);
                UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Decimator").Prefab.GetComponent<DecimatorBehaviour>().DeleteEffect, phys.transform.position, Quaternion.identity);
            }
             
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class LiquidBlackHole : Liquid
    {
        PhysicalBehaviour phys;

        public LiquidBlackHole()
        {
            Color = new Color(0f, 0f, 0f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {          
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container.gameObject.GetComponent<Doomsday>() == null)
            {
                container.gameObject.AddComponent<Doomsday>();
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {
        }
    }
    public class Doomsday : MonoBehaviour
    {
        PhysicalBehaviour phys;
        bool Doom = false;

        public void Awake()
        {
            Doom = false;
        }

        public void Update()
        {
            if(Doom == false)
            {
                phys = this.gameObject.GetComponent<PhysicalBehaviour>();
                if (phys.Temperature > Utils.CelsiusToPreference(-55f))
                {
                    Doom = true;
                    UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Decimator").Prefab.GetComponent<DecimatorBehaviour>().BlackHole, phys.transform.position, Quaternion.identity);
                }
            }
        }
    }


    public class StatuePoison : TemporaryBodyLiquid
    {
        public StatuePoison()
        {
            Color = new Color(0.35f, 0.35f, 0.35f);
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if(limb.SpeciesIdentity == "ANDROID")
                return;
                limb.CirculationBehaviour.StartCoroutine(this.MedusaEffect(limb));
        }

        public override void OnUpdate(BloodContainer container) 
        {
            if (container is CirculationBehaviour circ)
            {
                if (circ.Limb.NodeBehaviour.IsConnectedToRoot)
                    circ.Limb.Person.AddPain(1f);
            }
        }

        public override void OnEnterContainer(BloodContainer container) 
        {
        }

        public override void OnExitContainer(BloodContainer container) 
        {
        }

        public System.Collections.IEnumerator MedusaEffect(LimbBehaviour limb)
        {
            yield return (object) new WaitForSeconds(10f);
            var phys = limb.gameObject.GetComponent<PhysicalBehaviour>();
            var properties = phys.Properties.ShallowClone();
            phys.Properties = properties;

            limb.gameObject.GetComponent<SpriteRenderer>().color = new Color(0.35f, 0.35f, 0.35f, 1f);
            phys.Properties = ModAPI.FindPhysicalProperties("Concrete");
            limb.Frozen = true;
            limb.CirculationBehaviour.IsPump = false;
            limb.ImmuneToDamage = true;
            phys.Disintegratable = false;
            limb.BreakingThreshold = 10000f;
            properties.Flammability = 0f;
            properties.Burnrate = 0f;
            phys.TrueInitialMass *= 2f;
        }
    }

    public class UltraStrengthSerumPlus : Liquid
    {
        public UltraStrengthSerumPlus()
        {
            Color = new Color(0.54f, 0.44f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if (limb.SpeciesIdentity == "Android")
                return;
            limb.Vitality *= 0.05f;
            limb.RegenerationSpeed += 10f;
            limb.ImpactPainMultiplier *= 0.05f;
            limb.InitialHealth += 300f;
            limb.Health = limb.InitialHealth;
            limb.BreakingThreshold += 20f;
            limb.BaseStrength = Mathf.Min(30f, limb.BaseStrength + 10f);
            limb.ShotDamageMultiplier *= 0.05f;
            limb.CirculationBehaviour.HealBleeding();
            limb.Wince(20f);
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class UltraWeaknessPoison : Liquid
    {
        public UltraWeaknessPoison()
        {
            Color = new Color(0.88f, 0.86f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if (limb.SpeciesIdentity == "Android")
                return;
            limb.Vitality *= 5f;
            limb.RegenerationSpeed *= 0.5f;
            limb.ImpactPainMultiplier *= 5f;
            limb.InitialHealth *= 0.5f;
            limb.Health = limb.InitialHealth;
            limb.BreakingThreshold *= 0.5f;
            limb.BaseStrength *= 0.75f;
            limb.ShotDamageMultiplier *= 5f;
            limb.Wince(50f);
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }

    public class ConventionalLiquid : Liquid
    {
        public ConventionalLiquid()
        {
            Color = new Color(1f, 0.5f, 0f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if(limb.SpeciesIdentity == "Android")
                return;
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Head)
                limb.Person.Consciousness *= 0.85f;
            limb.BaseStrength = 0.75f;
            limb.Health *= 0.5f;
            limb.ShotDamageMultiplier *= 1.5f;
            limb.BreakingThreshold *= 0.8f;
            limb.RegenerationSpeed *= 0.5f;
            limb.ImpactPainMultiplier *= 1.5f;
            limb.CirculationBehaviour.BloodFlow *= 0.75f;
            if(UnityEngine.Random.value < 0.05f)
            {
                limb.CirculationBehaviour.IsPump = false;
            }

        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }
}