using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class Poison : TemporaryBodyLiquid
    {
        public Poison()
        {
            Color = new Color(0.3f, 0.75f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<Poisoning>();
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {           
        }
    }
    public class Poisoning : MonoBehaviour
    {
        LimbBehaviour limb;
        Poison poison;
        public float poisonPotency = 100f;
        float concentration;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }

        public void Update()
        {
            if(limb.SpeciesIdentity == "Android")
                return;
            concentration = limb.CirculationBehaviour.GetAmount(Liquid.GetLiquid("POISON")) * 10f;
            limb.Health -= 0.01f * poisonPotency * concentration * Time.deltaTime;
            poisonPotency -= 0.1f * Time.deltaTime;
        }
    }

    public class ConfusionAgent : Liquid
    {
        public ConfusionAgent()
        {
            Color = new Color(UnityEngine.Random.Range(0f,1f), UnityEngine.Random.Range(0f,1f), UnityEngine.Random.Range(0f,1f)); //random color to confuse >:)
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if (limb.SpeciesIdentity == "Android")
                return;
            if(limb.RoughClassification == LimbBehaviour.BodyPart.Head)
                limb.Person.Consciousness *= 0.75f;
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class RottingAgent : TemporaryBodyLiquid
    {
        public RottingAgent()
        {
            Color = new Color(0.79f, 0.78f, 0.41f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if (container is CirculationBehaviour circ)
            {
                if (circ.Limb.SpeciesIdentity == "Android")
                    return;
                float concentration = circ.Limb.CirculationBehaviour.GetAmount((Liquid) this) * 10f;
                circ.Limb.SkinMaterialHandler.RottenProgress += 1f * Time.deltaTime * concentration;
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class BreathingSerum : Liquid
    {
        public BreathingSerum()
        {
            Color = new Color(0.21f, 0.57f, 0.96f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if (!(container is CirculationBehaviour circ))
                return;
            LimbBehaviour limb = circ.Limb;
            if (limb.SpeciesIdentity == "Android" || limb.RoughClassification != LimbBehaviour.BodyPart.Head)
                return;
            limb.Person.OxygenLevel = 1f;
        }
    }

    public class SuffocationAgent : TemporaryBodyLiquid
    {
        public SuffocationAgent()
        {
            Color = new Color(0f, 0.51f, 0.51f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            base.OnUpdate(container);
            if (!(container is CirculationBehaviour circulationBehaviour))
                return;
            LimbBehaviour limb = circulationBehaviour.Limb;
            if (limb.SpeciesIdentity == "Android" || limb.RoughClassification != LimbBehaviour.BodyPart.Head)
                return;
            limb.Person.OxygenLevel -= 0.1f;
        }
    
        public override void OnExitContainer(BloodContainer container)
        {          
        }
    }

    public class TissueReconstructionAgent : TemporaryBodyLiquid
    {
        public TissueReconstructionAgent()
        {
            Color = new Color(0.38f, 0.56f, 0.28f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.Health = limb.InitialHealth;
            if(limb.RoughClassification != LimbBehaviour.BodyPart.Head)
                return;
            limb.Person.Consciousness = 1f;
            limb.Person.ShockLevel = 0.0f;
            limb.Person.PainLevel = 0.0f;
            limb.Person.OxygenLevel = 1f;
        }

        public override void OnUpdate(BloodContainer container)
        {
            if (container is CirculationBehaviour circ)
            {
                if (circ.Limb.SpeciesIdentity == "Android")
                    return;
                float concentration = circ.Limb.CirculationBehaviour.GetAmount((Liquid) this) * 10f;
                circ.Limb.PhysicalBehaviour.BurnProgress -= 1f * Time.deltaTime * concentration;
                circ.Limb.SkinMaterialHandler.AcidProgress -= 1f * Time.deltaTime * concentration;
            }
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }
    }

    public class Antivenom : Liquid
    {
        public Antivenom()
        {
            Color = new Color(0.7f, 0.75f, 1f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            if(limb.gameObject.GetComponent<Poisoning>() == null || limb.gameObject.GetComponent<Poisoning>().poisonPotency < 50f)
                return;
            limb.Health = limb.InitialHealth;
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<Poisoning>());
        }

        public override void OnUpdate(BloodContainer container)
        {
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class DerotifierSerum : Liquid
    {
        public DerotifierSerum()
        {
            Color = new Color(0.21f, 0.22f, 0.59f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if (container is CirculationBehaviour circ)
            {
                if (circ.Limb.SpeciesIdentity == "Android")
                    return;
                float concentration = circ.Limb.CirculationBehaviour.GetAmount((Liquid) this) * 10f;
                circ.Limb.SkinMaterialHandler.RottenProgress -= 1f * Time.deltaTime * concentration;
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class PainkillerAgent : TemporaryBodyLiquid 
    {
        public PainkillerAgent()
        {
            Color = new Color(0.5f, 1f, 0.73f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if(container is CirculationBehaviour circ)
            {
                circ.Limb.Numbness = 0.0f;
                if (circ.Limb.RoughClassification == LimbBehaviour.BodyPart.Head)
                {
                    circ.Limb.Person.ShockLevel = 0.0f;
                    circ.Limb.Person.PainLevel = 0.0f;
                }
            }
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }

    public class PainkillerSerum : TemporaryBodyLiquid   //scientists are bad at naming things :/
    {
        public PainkillerSerum()
        {
            Color = new Color(0f, 1f, 0.73f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {            
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.gameObject.AddComponent<NoPainInTheAss>();
            UnityEngine.Object.Destroy(limb.gameObject.GetComponent<PainInTheAss>());
        }

        public override void OnUpdate(BloodContainer container)
        {
            
        }
    
        public override void OnExitContainer(BloodContainer container)
        {            
        }
    }
    
    public class NoPainInTheAss : MonoBehaviour
    {
        LimbBehaviour limb;
        public void Awake()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
        }

        public void Update()
        {
            limb.Numbness = 0.0f;
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Head)
            {
                limb.Person.ShockLevel = 0.0f;
                limb.Person.PainLevel = 0.0f;
            }
        }
    }
}